package cn.com.taiji.mqcrj.mapper;

import org.apache.ibatis.annotations.Mapper;

/**
 * @author chen mh
 * @date 2023/7/26 15:45
 */
@Mapper
public interface BaseCreateNewTableMapper {

    /**
     * 创建数据库表
     *
     * @param tableName 表名称
     */
    void createTableAjxyr(String tableName);

    void createTableMqwgr(String tableName);

    void createTableAjxx(String tableName);

    /**
     * 修改表名
     * @param newTable
     * @param oldTable
     */
    void modifyTableName(String newTable,String oldTable);


    void dropTable(String tableName);

    void delTable(String tableName);
}
